Package: BiocFHIR
Title: Illustration of FHIR ingestion and transformation using R
Version: 1.14.0
Date: 2022-11-29
Authors@R: 
    c(person(
        "Vincent", "Carey", role = c("aut", "cre"),
        email = "stvjc@channing.harvard.edu",
        comment = c(ORCID = "0000-0003-4046-0063")
    ))
Description: FHIR R4 bundles in JSON format are derived from https://synthea.mitre.org/downloads.
	Transformation inspired by a kaggle notebook published by Dr
	Alexander Scarlat, 
	https://www.kaggle.com/code/drscarlat/fhir-starter-parse-healthcare-bundles-into-tables.
	This is a very limited illustration of some basic parsing and reorganization
	processes.  Additional tooling will be required to move beyond the Synthea data illustrations.
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.2)
Imports: DT, shiny, jsonlite, graph, tidyr, visNetwork, dplyr, utils,
        methods, BiocBaseUtils
Suggests: knitr, testthat, rjsoncons, igraph, BiocStyle
VignetteBuilder: knitr
biocViews: Infrastructure, DataImport, DataRepresentation
RoxygenNote: 7.3.2
URL: https://github.com/vjcitn/BiocFHIR
BugReports: https://github.com/vjcitn/BiocFHIR/issues
git_url: https://git.bioconductor.org/packages/BiocFHIR
git_branch: RELEASE_3_23
git_last_commit: ea4fb87
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:53:00 UTC; biocbuild
Author: Vincent Carey [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4046-0063>)
Maintainer: Vincent Carey <stvjc@channing.harvard.edu>

# Generated by roxygen2: do not edit by hand

S3method(print,BiocFHIR.FHIRgraph)
S3method(print,FHIR.bundle)
export(FHIR_ResourceTypes)
export(FHIR_retention_schemas)
export(FHIRtabs)
export(add_procedures)
export(available_retention_schemas)
export(build_proccond_igraph)
export(display_proccond_igraph)
export(freq_app)
export(getHumanName)
export(make_condition_graph)
export(make_test_json_set)
export(process_AllergyIntolerance)
export(process_CarePlan)
export(process_Claim)
export(process_Condition)
export(process_Encounter)
export(process_Immunization)
export(process_MedicationRequest)
export(process_Observation)
export(process_Patient)
export(process_Procedure)
export(process_fhir_bundle)
export(stack_fhir)
export(summarise_bundles)
export(summary_selections)
import("graph", except=c("union", "edges", "intersection", "degree"))
import(dplyr)
import(shiny)
importFrom(BiocBaseUtils,selectSome)
importFrom(jsonlite,fromJSON)
importFrom(methods,new)
importFrom(tidyr,unnest)
importFrom(utils,data)
importFrom(utils,packageDescription)
importFrom(utils,unzip)
importFrom(visNetwork,visIgraph)

# BiocFHIR

Illustration of FHIR V4 Processing with synthea data


This R package addresses very basic tasks of parsing FHIR R4 documents in JSON format.

We will work through elements of the following concept map:

![coggle](https://github.com/vjcitn/BiocFHIR/raw/main/conceptmap.jpg)

# Use in AnVIL

In a terra workspace 

- use an Rstudio cloud environment
- start a git project
- use https://github.com/vjcitn/BiocFHIR as the repository
- in the Rstudio console 
    - use `BiocManager::install("vjcitn/BiocFHIR")` to get the software
    - navigate to the vignettes folder and knit the Rmd file to HTML

This is a collection of 30 FHIR records in JSON format.

The data were retrieved from https://synthetichealth.github.io/synthea-sample-data/downloads/synthea_sample_data_fhir_r4_sep2019.zip

The downloads page includes:

"Data hosted within SyntheticMass has been generated by SyntheaTM, an open-source patient population simulation made available by The MITRE Corporation.

The data is free from cost, privacy, and security restrictions. It can be used without restriction for a variety of secondary uses in academia, research, industry, and government.

Please cite SyntheaTM or SyntheticMass as:

Jason Walonoski, Mark Kramer, Joseph Nichols, Andre Quina, Chris Moesel, Dylan Hall, Carlton Duffett, Kudakwashe Dube, Thomas Gallagher, Scott McLachlan, Synthea: An approach, method, and software mechanism for generating synthetic patients and the synthetic electronic health care record, Journal of the American Medical Informatics Association, Volume 25, Issue 3, March 2018, Pages 230–238, https://doi.org/10.1093/jamia/ocx079"

#' convert data of a given FHIR type in a list of bundles to a data.frame
#' @param blist list of FHIR bundles imported with process_fhir_bundle
#' @param type character(1) type, in names(blist[[1]]), e.g.
#' @param droperr logical(1) exclude records for which process_[type] fails, defaults to TRUE
#' @return data.frame
#' @examples
#' jj = make_test_json_set()
#' b2 = lapply(jj[1:2], process_fhir_bundle)
#' ss = stack_fhir(b2, "Procedure")
#' head(ss,2)
#' @export
stack_fhir = function(blist, type, droperr=TRUE) {
   if (type == "Patient") return(get_patient_stack(blist)) # Patient recs too heterogenous
   func = get(paste0("process_", type))
   pulls = lapply(blist, function(x) try(func(x[[type]]),silent=TRUE))
   haserr = sapply(pulls, function(x) inherits(x, "try-error"))
   if (any(haserr)) {
     message(sprintf("...bundle %d missing a component\n", which(haserr)))
     pulls = pulls[-which(haserr)]
     }
   ans = do.call(rbind, pulls)
   bad = grep("Error in func", ans[,1]) # issue with AllergyIntolerance
   if (length(bad)>0) {
     ans = ans[-bad,]
     }
   ans
}

get_patient_stack = function(blist) {
  pp = lapply(blist, function(x) process_Patient(x$Patient)) # produces single column
# extract intersection of all attributes
  props = rownames(pp[[1]])
  for (i in seq_len(length(pp)))  # first intersection nugatory
     props <- intersect(props, rownames(pp[[i]]))
  gg = lapply(pp, function(x) t(x[props,]))
  dd = data.frame(do.call(rbind,gg))
  names(dd) = props
  dd
}


#process_CarePlan = function (CarePlan) 
#{
#    stopifnot(inherits(CarePlan, "BiocFHIR.CarePlan"))
#    ac <- CarePlan$activity
#    recc <- vapply(ac, nrow, numeric(1))
#    idvec <- rep(CarePlan$id, recc)
#    getdc <- function(x) do.call(rbind, x$detail$code$coding)
#    ac_coding <- do.call(rbind, lapply(ac, getdc))
#    ac_status <- do.call(c, lapply(ac, function(x) x$detail$status))
#    ac_loc <- unname(unlist(lapply(ac, function(x) x$detail$location)))
##
## in synthea data, categories can be more complex; if there are two rows per entry,
## the second will have useful textual information, first will have a 'care plan' code
##
##    cattext = lapply(CarePlan$category, function(x) as.character(na.omit(x$text)))
#    last = function(x) x[length(x),]
#    cattext = lapply(CarePlan$category, last)
##    ml <- lapply(CarePlan$category, function(x) x$coding)
#    ml = lapply(cattext, function(x) x$coding)
#    uu <- unlist(ml, recursive = FALSE)
#    catd <- do.call(rbind, rep(uu, recc))
#    subref <- CarePlan$subject$reference
#    subid <- rep(subref, recc)
#    data.frame(id = idvec, ac_coding, status = ac_status, location = ac_loc, 
#        category = catd, subject.reference = subid)
#}
#

library(testthat)
library(BiocFHIR)
test_that("ingest succeeds", {
 library(BiocFHIR)
 example(process_fhir_bundle)
 expect_true(length(names(tbun))>0)
})

myl = make_test_json_set()
if (!exists("bs")) bs = lapply(myl, process_fhir_bundle)


test_that("stack for CarePlan is correct",
 {
 ss = stack_fhir(bs, "CarePlan")
print(dim(ss))
print(head(ss))
 expect_true(nrow(ss)==231)
 expect_true(ncol(ss)==10)
 })

test_that("stack for Claim is correct",
 {
 ss = stack_fhir(bs, "Claim")
print(dim(ss))
 expect_true(nrow(ss)==1423)
 expect_true(ncol(ss)==9)
 })

test_that("stack for Condition is correct",
 {
 ss = stack_fhir(bs, "Condition")
print(dim(ss))
 expect_true(nrow(ss)==247)
 expect_true(ncol(ss)==5)
 })

test_that("stack for Encounter is correct",
 {
 ss = stack_fhir(bs, "Encounter")
print(dim(ss))
# print(dim(ss))
 expect_true(TRUE)
 expect_true(nrow(ss)==1032)
 expect_true(ncol(ss)==9)
 })

test_that("stack for Immunization is correct",
 {
 ss = stack_fhir(bs, "Immunization")
print(dim(ss))
 expect_true(nrow(ss)==406)
 expect_true(ncol(ss)==6)
 })

test_that("stack for MedicationRequest is correct",
 {
 ss = stack_fhir(bs, "MedicationRequest")
print(dim(ss))
 expect_true(nrow(ss)==391)
 expect_true(ncol(ss)==8)
 })


test_that("stack for Observation is correct",
 {
 ss = stack_fhir(bs, "Observation")
print(dim(ss))
 expect_true(nrow(ss)==6860)
 expect_true(ncol(ss)==11)
 })

test_that("stack for Patient is correct",
 {
 ss = stack_fhir(bs, "Patient")
 print(dim(ss))
 expect_true(TRUE)
 expect_true(nrow(ss)==30)
 expect_true(ncol(ss)==40)
 })

test_that("stack for Procedure is correct",
 {
 ss = stack_fhir(bs, "Procedure")
print(dim(ss))
 expect_true(nrow(ss)==881)
 expect_true(ncol(ss)==8)
 })

#> names(bs[[1]])
#[1] "CarePlan"          "Claim"             "Condition"        
#[4] "Encounter"         "Immunization"      "MedicationRequest"
#[7] "Observation"       "Patient"           "Procedure"        
#example(process_Claim)
#ss = stack_fhir(bs, "Claim")
#example(process_Condition)
#ss = stack_fhir(bs, "Condition")
#example(process_Encounter)
#ss = stack_fhir(bs, "Encounter")
#example(process_Immunization)
#ss = stack_fhir(bs, "Immunization")
#example(process_MedicationRequest)
#ss = stack_fhir(bs, "MedicationRequest")
#example(process_Observation)
#ss = stack_fhir(bs, "Observation")
#example(process_Patient)
#ss = stack_fhir(bs, "Patient")
#example(process_Procedure)
#ss = stack_fhir(bs, "Procedure")

#' extract information from retained fields in Condition component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Condition component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Condition(tbun$Condition)
#' @export
process_Condition <- function(Condition) {
 stopifnot(inherits(Condition, "BiocFHIR.Condition"))
 coding <- do.call(rbind, Condition$code$coding)
 data.frame(id=Condition$id,subject.reference=Condition$subject$reference, code.coding=coding)
}

#' produce 50 json FHIR files in a folder
#' @importFrom utils unzip
#' @param target character(1) a path, defaults to `jsontest` under `tempdir()`; the
#' contents of synthfhir.zip, in inst/zip of BiocFHIR, will be deposted there.
#' @param reuse logical(1) if TRUE, just use what is there, if folder already exists
#' @return a vector of paths to FHIR JSON, invisibly
#' @examples
#' z <- make_test_json_set()
#' z[1:3]
#' @export
make_test_json_set <- function(target=paste0(tempdir(), "/jsontest"), reuse=TRUE) {
  if (dir.exists(target)) {
    if (!reuse) stop("target folder already exists")
    }
  else dir.create(target)
  unzip(system.file("zip/synthfhir.zip", package="BiocFHIR"), exdir=target)
  ans <- dir(target, full.names=TRUE, pattern="json$")
  invisible(ans)
}
  
  

#' extract information from retained fields in Claim component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Claim component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Claim(tbun$Claim)
#' @export
process_Claim <- function(Claim) {
 stopifnot(inherits(Claim, "BiocFHIR.Claim"))
 ins <- Claim$insurance
 #insdf <- do.call(rbind, lapply(ins, tidyr::unnest, cols=c()))
 insdf = vapply(ins, function(x) x$coverage$display, character(1))
 data.frame(id=Claim$id,patient=Claim$patient,provider=Claim$provider, insurance=insdf,
     billablePeriod=Claim$billablePeriod, created=Claim$created)
}

#' extract information from retained fields in Encounter component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Encounter component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Encounter(tbun$Encounter)
#' @export
process_Encounter <- function(Encounter) {
  stopifnot(inherits(Encounter, "BiocFHIR.Encounter"))
  co = Encounter$type
  if (inherits(co, "data.frame")) type = Encounter$type
      else type <- do.call(rbind, lapply(Encounter$type, function(x) (x$coding[[1]])))
  subj <- Encounter$subject$reference
  prov <- Encounter$serviceProvider
  per <- Encounter$period
  data.frame(id=Encounter$id, type=type, subject=subj, serviceProvider=prov,
    period=per)
}

#' extract information from retained fields in AllergyIntolerance component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param AllergyIntolerance component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_AllergyIntolerance(tbun$AllergyIntolerance)
#' @export
process_AllergyIntolerance <- function(AllergyIntolerance) {
 stopifnot(inherits(AllergyIntolerance, "BiocFHIR.AllergyIntolerance"))
 coding <- do.call(rbind, AllergyIntolerance$code$coding)
 data.frame(id=AllergyIntolerance$id, onsetDateTime=AllergyIntolerance$onsetDateTime,
   patient.reference=AllergyIntolerance$patient$reference,
   category=unlist(AllergyIntolerance$category), code=coding)
}

# extract information from retained fields in Observation component of FHIR Bundle, produce simple data.frame
# @importFrom tidyr unnest
# @param Observation component of FHIR.bundle instance
# @return data.frame
# @examples
# testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#    package="BiocFHIR")
# tbun <- process_fhir_bundle(testf)
# process_Observation(tbun$Observation)
# @export
#process_Observation <- function(Observation) {
# stopifnot(inherits(Observation, "BiocFHIR.Observation"))
# coding <- do.call(rbind, Observation$code$coding)
# data.frame(id=Observation$id, subject.reference=Observation$subject$reference, code.coding=coding, valueQuantity=Observation$valueQuantity, effectiveDateTime=Observation$effectiveDateTime, issued=Observation$issued)
#}

#' extract information from retained fields in MedicationRequest component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param MedicationRequest component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_MedicationRequest(tbun$MedicationRequest)
#' @export
process_MedicationRequest <- function(MedicationRequest) {
  stopifnot(inherits(MedicationRequest, "BiocFHIR.MedicationRequest"))
  mr <- MedicationRequest
  mrco <- do.call(rbind, mr$medicationCodeableConcept$coding)
  data.frame(id=MedicationRequest$id, subject=mr$subject$reference, status=mr$status, requester=mr$requester, medicationCodeableConcept=mrco)
}

#' extract information from retained fields in Procedure component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Procedure component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Procedure(tbun$Procedure)
#' @export
process_Procedure <- function(Procedure) {
  stopifnot(inherits(Procedure, "BiocFHIR.Procedure"))
  pro <- Procedure
  coding <- do.call(rbind, Procedure$code$coding)
  data.frame(id=pro$id,subject=pro$subject$reference, status=pro$status, performedPeriod=pro$performedPeriod, 
     code=coding)
}

#' extract information from retained fields in Immunization component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Immunization component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Immunization(tbun$Immunization)
#' @export
process_Immunization <- function(Immunization) {
  stopifnot(inherits(Immunization, "BiocFHIR.Immunization"))
  pro <- Immunization
  coding <- do.call(rbind, Immunization$vaccineCode$coding)
  data.frame(id=pro$id, patient=pro$patient$reference, vaccineCode=coding, occurrenceDateTime=pro$occurrenceDateTime)
}

#' flatten information in Patient component of a bundle to a one-line data.frame
#' @param Patient element of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' tpat <- process_Patient(tbun$Patient)
#' head(names(tpat))
#' tags <- c("identifier.system3", "identifier.value3")
#' tpat[tags,,FALSE]
#' tags2 <- grep("extension.extension", rownames(tpat), value=TRUE)
#' tpat[tags2,,FALSE]
#' @export
process_Patient <- function(Patient) {
 # Motivation: Patient bundle component has 'identifier' which
 # comes as a list of length 1, which has embedded data.frame
# Example
# > str(origpat$identifier)
# List of 1
#  $ :'data.frame':	3 obs. of  3 variables:
#   ..$ system: chr [1:3] "https://github.com/synthetichealth/synthea" "http://hospital.smarthealthit.org" "http://hl7.org/fhir/sid/us-ssn"
#   ..$ value : chr [1:3] "6fa3d4ab-c0b6-424a-89d8-7d9105129296" "6fa3d4ab-c0b6-424a-89d8-7d9105129296" "999-91-5546"
#   ..$ type  :'data.frame':	3 obs. of  2 variables:
#   .. ..$ coding:List of 3
#   .. .. ..$ : NULL
#   .. .. ..$ :'data.frame':	1 obs. of  3 variables:
#   .. .. .. ..$ system : chr "http://terminology.hl7.org/CodeSystem/v2-0203"
#   .. .. .. ..$ code   : chr "MR"
#   .. .. .. ..$ display: chr "Medical Record Number"
#   .. .. ..$ :'data.frame':	1 obs. of  3 variables:
#   .. .. .. ..$ system : chr "http://terminology.hl7.org/CodeSystem/v2-0203"
#   .. .. .. ..$ code   : chr "SS"
#   .. .. .. ..$ display: chr "Social Security Number"
#   .. ..$ text  : chr [1:3] NA "Medical Record Number" "Social Security Number"
#
# we want to flatten this, but the NULL for coding for the first system is a bit
# of a problem.  Simple solution is to unlist all Patient components
#
  stopifnot(inherits(Patient, "BiocFHIR.Patient"))
  ans <- do.call(data.frame, lapply(Patient, function(x) data.frame(t(unlist(x)))))
  ans <- data.frame(t(ans))
  names(ans) <- c("value")
  rownames(ans)[1] = "id"
  ans
}

#' process a bundle of FHIR R4 JSON
#' @importFrom jsonlite fromJSON
#' @importFrom BiocBaseUtils selectSome
#' @param json_file character(1) path to text in JSON format
#' @param schemas list of character vectors defining expected fields, defaults to FHIR_retention_schemas()
#' @return instance of FHIR.bundle, extending list
#' @note If one encounters the error "Element ... lacks field", the schemas argument can be modified by removing
#' the noted field from the schema.
#' @examples
#' testf = system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun = process_fhir_bundle(testf)
#' tbun
#' @export
process_fhir_bundle = function(json_file, schemas = FHIR_retention_schemas()) {
  dat = fromJSON(json_file)
  stopifnot(all(c("resourceType", "type", "entry") %in% names(dat)))
  stopifnot(dat$resourceType == "Bundle")
  de = dat$entry
  spl = split(de$resource, de$resource$resourceType)
  nres = names(spl)
  ok = intersect(nres, available_retention_schemas())
  scs = schemas
  check_bundle_restype = function(restype, splitbundle, schemas) {
   spln = names(splitbundle[[restype]])
   scsn = schemas[[restype]]
   data_lacks_field = setdiff(scsn, spln)
   if (length(data_lacks_field)>0) {
     message(sprintf("Element %s lacks field %s; retention schema must be modified to process this bundle.\n",
            restype, data_lacks_field))
     stop("Can't process bundle using existing retention schemas")
     }
   splitbundle[[restype]][,schemas[[restype]],drop=FALSE]
   }
  ans = lapply(ok, function(x) check_bundle_restype(x, spl, scs)) # fails if a field is missing
  names(ans) = ok
  ans = lapply(ok, function(x) {class(ans[[x]]) = c(paste0("BiocFHIR.", x), class(ans[[x]])); ans[[x]]})
  names(ans) = ok # lapply does not propagate by default
  class(ans) = c("FHIR.bundle", "list")
  ans
}

#fixup_Patient = function(dfr) {
#  colclasses = vapply(dfr, class, character(1))
#  is_df = which(colclasses == "data.frame")
#  is_list = which(colclasses == "list")
#  ok = setdiff(seq_len(ncol(dfr)), c(is_df, is_list))
#  newd = dfr[,ok,drop=FALSE]
#  needsu = dfr[,is_df,drop=FALSE]
#  better = lapply(needsu, unnest, cols=c())
#  cbind(newd, needsu, better)
#}
  

#do_unnest = function(dfr) {
#  ok = setdiff(seq_len(ncol(dfr)), c(is_df, is_list))
#  newd = dfr[,ok,drop=FALSE]
#  better = lapply(needsu, unnest, cols=c())
#  fixl = lapply(dfr[,is_list,drop=FALSE], "[[", 1)
#  bfix = lapply(fixl, unnest, cols=c())
#  do.call(cbind, c(newd, better, bfix))
#}

#' print method
#' @param x BiocFHIR FHIR.bundle instance
#' @param \dots not used
#' @return print method
#' @export
print.FHIR.bundle = function(x, ...) {
  cat("BiocFHIR FHIR.bundle instance.\n")
  cat("  resource types are:\n")
  cat("  ", BiocBaseUtils::selectSome(names(x)))
  cat("\n")
}



#' list available 'retention schemas'
#' @return character vector
#' @examples
#' available_retention_schemas()
#' @export
available_retention_schemas = function()
 names(FHIR_retention_schemas())

#1: replacing previous import 'graph::union' by 'igraph::union' when loading 'BiocFHIR' 
#2: replacing previous import 'graph::edges' by 'igraph::edges' when loading 'BiocFHIR' 
#3: replacing previous import 'graph::intersection' by 'igraph::intersection' when loading 'BiocFHIR' 
#4: replacing previous import 'graph::degree' by 'igraph::degree' when loading 'BiocFHIR' 


#' get human name from a BiocFHIR.Patient instance
#' @importFrom methods new
#' @rawNamespace import("graph", except=c("union", "edges", "intersection", "degree"))
#' @param Patient BiocFHIR.Patient instance
#' @return string with name components concatenated
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' getHumanName(tbun$Patient)
#' @export
getHumanName <- function(Patient) { 
   stopifnot(inherits(Patient, "BiocFHIR.Patient"))
   ul <- unlist(Patient);   # unpleasant to avoid "process_Patient" but simpler?
   if (!is.na(ul["name.family1"]))
     return(paste0(ul["name.family1"], ul["name.family2"]))
   else  return(paste0(ul["name.given"], ul["name.family"]))
}

#' create graph with links from patients to conditions
#' @param listOfProcessedBundles list with elements generated by `process_fhir_bundle`
#' @param keep_with_condition_only logical(1) omit bundles that lack a "Condition" element, defaults to TRUE
#' @return instance of BiocFHIR.FHIRgraph
#' @examples
#' data("allin")
#' make_condition_graph(allin, TRUE)
#' @export
make_condition_graph <- function(listOfProcessedBundles, keep_with_condition_only=TRUE) {

 patients <- vapply(listOfProcessedBundles, function(x) getHumanName(x$Patient), character(1))
 condg <- new("graphNEL", nodes=patients)
 condg@graphData <- list(edgemode="directed")

 if (keep_with_condition_only) {
   alln = lapply(listOfProcessedBundles, names)
   hascond = vapply(alln, function(x) "Condition" %in% x, logical(1))
   if (!all(hascond)) listOfProcessedBundles = listOfProcessedBundles[which(hascond)]
   }

 nn <- lapply( listOfProcessedBundles, function(x) {
    curn <- getHumanName(x$Patient)
    curconds <- try(process_Condition(x$Condition),silent=TRUE) #$code.coding.display)
    if (inherits(curconds, "try-error")) return(curconds)
    curconds <- curconds$code.coding.display
#    if (!is.null(curconds)) {
    newnodes <- setdiff(curconds, nodes(condg))
    if (length(newnodes)>0) {
      condg <- addNode(newnodes, condg)
      }
   condg <<- addEdge( curn, curconds, condg)  # must use current conditions
   condg
#    }
    })
 errs <- vapply(nn, function(x) inherits(x, "try-error"), logical(1))
 haserr <- sum(errs)
 if (haserr > 0) message("...some bundles had no Condition component")
 conditions <- setdiff(nodes(condg), patients) # hokey
 ans <- list(graph=condg, patients=patients, conditions=conditions)
 class(ans) <- "BiocFHIR.FHIRgraph"
 ans
}

#' show a combination of graph and patient attributes
#' @param x instance of BiocFHIR.FHIRgraph
#' @param \dots not used
#' @return print method
#' @examples
#' data("allin")
#' make_condition_graph(allin)
#' @export
print.BiocFHIR.FHIRgraph <- function(x, ...) {
 cat("BiocFHIR.FHIRgraph instance.\n")
 print(x$graph)
 cat(sprintf(" %d patients, %d conditions\n", length(x$patients), length(x$conditions)))
}


#' update a fhir graph on patients and conditions with procedures
#' @param fhirgraph instance of BiocFHIR.FHIRgraph
#' @param listOfProcessedBundles list
#' @return instance of BiocFHIR.FHIRgraph
#' @examples
#' data("allin")
#' g <- make_condition_graph(allin)
#' g <- add_procedures(g, allin)
#' g
#' @export
add_procedures <- function(fhirgraph, listOfProcessedBundles) {
 stopifnot(inherits(fhirgraph, "BiocFHIR.FHIRgraph"))
 curg <- fhirgraph$graph
 nn <- lapply( listOfProcessedBundles, function(x) {
    curn <- getHumanName(x$Patient)
    curprocs <- try(process_Procedure(x$Procedure),silent=TRUE) # $code.display)
    if (inherits(curprocs, "try-error")) return(curprocs)
    curprocs <- curprocs$code.display
    newnodes <- setdiff(curprocs, nodes(curg))
    if (length(newnodes)>0) {
        curg <- addNode(newnodes, curg)
        }
   curg <<- addEdge( curn, curprocs, curg)
    curg
    })
 errs <- vapply(nn, function(x) inherits(x, "try-error"), logical(1))
 haserr <- sum(errs)
 if (haserr > 0) message("...some bundles had no Procedure component")
 fhirgraph$procedures <- setdiff(nodes(curg), nodes(fhirgraph$graph))
 fhirgraph$graph <- curg
 fhirgraph
}

#library(BiocFHIR)
#library(graph)
#library(parallel)
#library(igraph)
#options(mc.cores=4)
##allin <- mclapply(dir(pattern="json$"), process_fhir_bundle)
#load("allin.rda")
#condg <- make_condition_graph( allin )
#condg <- add_procedures( condg, allin )
#condg
#
#ii <- igraph.from.graphNEL(condg$graph)



#' build graph with patients, conditions and procedures
#' @param listOfBundles list of processed FHIR bundles, processed with `process_fhir_bundle`
#' @return instance of visIgraph from visNetworks
#' @examples
#' data("allin")
#' build_proccond_igraph( allin )
#' @export
build_proccond_igraph <- function( listOfBundles ) {
 condg <- make_condition_graph( listOfBundles )
 condg <- add_procedures( condg, listOfBundles )
 ii <- igraph::igraph.from.graphNEL( condg$graph )
 igraph::V(ii)$color[ names(igraph::V(ii)) %in% condg$conditions ] <- "red"
 igraph::V(ii)$color[ names(igraph::V(ii)) %in% condg$procedures ] <- "green"
 igraph::V(ii)$color[ names(igraph::V(ii)) %in% condg$patients ] <- "blue"
 ii
}

#' make network visualization
#' @importFrom visNetwork visIgraph
#' @param igraph instance of igraph produced by `build_procond_igraph`
#' @return visIGraph instance
#' @examples
#' data("allin")
#' g <- build_proccond_igraph( allin ) 
#' if (interactive()) {
#'  display_proccond_igraph( g )
#' }
#' @export
display_proccond_igraph <- function( igraph ) {
 visIgraph( igraph )
}

#' table app
#' @import shiny
#' @importFrom utils data packageDescription
#' @return No value returned.
#' @examples
#' if (interactive()) {
#'  FHIRtabs()
#' }
#' @export
FHIRtabs <- function() {
  fns <- make_test_json_set()
  zn <- gsub(".*jsontest/", "", fns)
  zns <- strsplit(zn, "_")
  fn <- vapply(zns, function(x) x[1], character(1))
  ln <- vapply(zns, function(x) x[2], character(1))
  folks <- paste(fn, ln, sep="_")

  names(fns) <- folks
  ui <- fluidPage(
   sidebarLayout(
    sidebarPanel(
     helpText("FHIR viewer for synthea data subset"),
     selectInput("indiv", "Person", choices=fns), 
     actionButton("stopBtn", "stop app"),
     width=2
     ),
    mainPanel(
     tabsetPanel(
      tabPanel("patient",
       DT::dataTableOutput("patient")
       ),
      tabPanel("condition",
       DT::dataTableOutput("condition")
       ),
      tabPanel("encounter",
       DT::dataTableOutput("encounter")
       ),
      tabPanel("Rx",
       DT::dataTableOutput("Rx")
       ),
      tabPanel("observation",
       DT::dataTableOutput("observation")
       ),
      tabPanel("procedure",
       DT::dataTableOutput("procedure")
       ),
      tabPanel("careplan",
       DT::dataTableOutput("careplan")
       ),
      tabPanel("claims",
       DT::dataTableOutput("claims")
       ),
      tabPanel("network",
       helpText("This is a display of relations among patients, conditions, and procedures for 50 synthea bundles"),
       helpText("Use clickwheel to zoom in, click on to drag"),
       visNetwork::visNetworkOutput("network")
       ),
      tabPanel("about",
       helpText("FHIR documents were sampled from those distributed at 
https://synthetichealth.github.io/synthea-sample-data/downloads/synthea_sample_data_fhir_r4_sep2019.zip"),
       helpText("The downloads page includes:

'Data hosted within SyntheticMass has been generated by SyntheaTM, an open-source patient population simulation made available by The MITRE Corporation.

The data is free from cost, privacy, and security restrictions. It can be used without restriction for a variety of secondary uses in academia, research, industry, and government.

Please cite SyntheaTM or SyntheticMass as:

Jason Walonoski, Mark Kramer, Joseph Nichols, Andre Quina, Chris Moesel, Dylan Hall, Carlton Duffett, Kudakwashe Dube, Thomas Gallagher, Scott McLachlan, Synthea: An approach, method, and software mechanism for generating synthetic patients and the synthetic electronic health care record, Journal of the American Medical Informatics Association, Volume 25, Issue 3, March 2018, Pages 230-238, https://doi.org/10.1093/jamia/ocx079'"),
       helpText("Fields retrieved for FHIR concepts are determined by a 'schema' collection."),
       helpText("For some documents, it may be necessary to collect information from a different field
than that specified in these schemas.  For example, a document may use 'patient' or 'subject' to identify
an individual.  A 'fallback' schema usage may be adopted in future versions to cope with this."),
       verbatimTextOutput("schema"),
       verbatimTextOutput("pkgdesc")
       ),
      )
     )
    )
   ) # fluidPage

   server <- function(input, output) {
     fdat <- reactive({
      process_fhir_bundle(input$indiv)
      })
     output$patient <- DT::renderDataTable({
         process_Patient(fdat()$Patient)
         })
     output$observation <- DT::renderDataTable({
         process_Observation(fdat()$Observation)
         })
     output$claims <- DT::renderDataTable({
         process_Claim(fdat()$Claim)
         })
     output$condition <- DT::renderDataTable({
         process_Condition(fdat()$Condition)
         })
     output$encounter <- DT::renderDataTable({
         process_Encounter(fdat()$Encounter)
         })
     output$Rx <- DT::renderDataTable({
         process_MedicationRequest(fdat()$MedicationRequest)
         })
     output$procedure <- DT::renderDataTable({
         process_Procedure(fdat()$Procedure)
         })
     output$careplan <- DT::renderDataTable({
         process_CarePlan(fdat()$CarePlan)
         })
     output$schema <- renderPrint({
         BiocFHIR::FHIR_retention_schemas()
         })
     output$network <- visNetwork::renderVisNetwork({
         allin <- NULL
         data_store <- new.env(parent = emptyenv())
         data("allin", package="BiocFHIR", envir = data_store)
         allin <- data_store[["allin"]]
         BiocFHIR::FHIR_retention_schemas()
         g <- build_proccond_igraph( allin ) 
         display_proccond_igraph( g )
         })
     output$pkgdesc <- renderPrint({
         packageDescription("BiocFHIR")
         })
     observeEvent(input$stopBtn, {
       stopApp(returnValue=NULL)   # could return information here
      })

     }
    runApp(list(ui=ui, server=server))
   }

  
#' extract information from retained fields in CarePlan component of FHIR Bundle, produce simple data.frame
#' @param CarePlan component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_CarePlan(tbun$CarePlan)
#' @note Nov 13 2022, added code to refine the 'category' data processing.
#' @export
process_CarePlan = function (CarePlan) 
{
    stopifnot(inherits(CarePlan, "BiocFHIR.CarePlan"))
    ac <- CarePlan$activity
    recc <- vapply(ac, nrow, numeric(1))
    idvec <- rep(CarePlan$id, recc)
    getdc <- function(x) do.call(rbind, x$detail$code$coding)
    ac_coding <- do.call(rbind, lapply(ac, getdc))
    ac_status <- do.call(c, lapply(ac, function(x) x$detail$status))
    ac_loc <- unname(unlist(lapply(ac, function(x) x$detail$location)))
#
# in synthea data, categories can be more complex; if there are two rows per entry,
# the second will have useful textual information, first will have a 'care plan' code
#
#    cattext = lapply(CarePlan$category, function(x) as.character(na.omit(x$text)))
    last = function(x) x[nrow(x),]
    cattext = lapply(CarePlan$category, last)
#    ml <- lapply(CarePlan$category, function(x) x$coding)
    ml = lapply(cattext, function(x) x$coding)
    uu <- unlist(ml, recursive = FALSE)
    catd <- do.call(rbind, rep(uu, recc))
    subref <- CarePlan$subject$reference
    subid <- rep(subref, recc)
    data.frame(id = idvec, ac_coding, status = ac_status, location = ac_loc, 
        category = catd, subject.reference = subid)
}


process_CarePlan_OLD <- function(CarePlan) {
# record count
 stopifnot(inherits(CarePlan, "BiocFHIR.CarePlan"))
 ac <- CarePlan$activity
 recc <- vapply(ac, nrow, numeric(1))
 idvec <- rep(CarePlan$id, recc)  # event ids
 getdc <- function(x) do.call(rbind, x$detail$code$coding)
 ac_coding <- do.call(rbind, lapply(ac, getdc))
 ac_status <- do.call(c, lapply(ac, function(x) x$detail$status))
 ac_loc <- unname(unlist(lapply(ac, function(x) x$detail$location)))
#
# process category
#
 ml <- lapply(CarePlan$category, function(x)x$coding)
 uu <- unlist(ml, recursive=FALSE) 
 catd <- do.call(rbind, rep(uu, recc))
#
 subref <- CarePlan$subject$reference
 subid <- rep(subref, recc)
 data.frame(id=idvec, ac_coding, status=ac_status, location=ac_loc, category=catd, subject.reference=subid)
}

#' FHIR Resource types recognized in package
#' @return vector of strings
#' @examples
#' FHIR_ResourceTypes() # Oct 2022
#' @export
FHIR_ResourceTypes = function() {
 c("AllergyIntolerance", "CarePlan", "CareTeam", 
  "Claim", "Condition", "DiagnosticReport", 
  "Encounter", "ExplanationOfBenefit", "Immunization", 
  "MedicationRequest", "Observation", "Organization", 
  "Patient", "Practitioner", "Procedure")
}

#' collection of FHIR Resource components to be retained
#' @return list of vectors of strings
#' @examples
#' FHIR_retention_schemas() # Oct 2022
#' @export
FHIR_retention_schemas = function() {
list(
  Condition = c("id", "onsetDateTime", "code", "subject"),
  AllergyIntolerance = c("id", "onsetDateTime", "code", "patient", "category"),
  CarePlan = c("id", "activity", "subject", "category"),
  Claim = c("id", "provider", "patient", "billablePeriod", "insurance", "created"),
  Encounter = c("id", "type", "subject", "period", "serviceProvider", "class"),
  MedicationRequest = c("id", "subject", "status", "requester", "medicationCodeableConcept"),
  Observation = c("id", "subject", "code", "valueQuantity", "category",
    "effectiveDateTime", "issued", "component"),  # blood pressures in component
  Procedure = c("id", "subject", "status", "performedPeriod", "code"),
  Patient = c("id", "identifier", "name", 
    "telecom", "gender", "birthDate", "address",   # dropped deceasedDateTime oct 2 2022
    "maritalStatus", "multipleBirthBoolean", "communication", "active"),
  Immunization = c("id", "patient", "vaccineCode", "occurrenceDateTime")
 )
}

## ----setup,results="hide",echo=FALSE------------------------------------------
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
library(rjsoncons)
})
})

## ----takepeek-----------------------------------------------------------------
tfile = dir(system.file("json", package="BiocFHIR"), full=TRUE)
peek = jsonlite::fromJSON(tfile)
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))

## ----lkcat--------------------------------------------------------------------
head(which(vapply(peek$entry$resource$category, 
   function(x)!is.null(x), logical(1))))
peek$entry$resource$category[[6]]
peek$entry$resource$category[[6]]$coding
peek$entry$resource$category[[10]]

## ----lkpeek2------------------------------------------------------------------
peek2 = jsonlite::fromJSON(dir(system.file("json", package="BiocFHIR"), full=TRUE), simplifyVector=FALSE)
lapply(peek2$entry[1:5], function(x) names(x[["resource"]]))

## ----lktab--------------------------------------------------------------------
rtyvec = vapply(peek2$entry, function(x) 
   x[["resource"]]$resourceType, character(1))
table(rtyvec)

## ----getcond------------------------------------------------------------------
iscond = which(rtyvec == "Condition")
conds = peek2$entry[iscond]
length(conds)
str(conds[[1]])

## ----chkb---------------------------------------------------------------------
tbu = process_fhir_bundle(tfile)
tbu

## ----lktab2-------------------------------------------------------------------
ctab = process_Condition(tbu$Condition)
dim(ctab)
datatable(ctab)

## ----dojme--------------------------------------------------------------------
z = make_test_json_set()
myl = lapply(z[1:4], jsonlite::fromJSON) # list that rconsjson will convert to JSON
library(rjsoncons)
tmp = jmespath(myl, "[*].entry[0].resource.address") |> jsonlite::fromJSON()
do.call(rbind,lapply(tmp, function(x) x[,-(1:2)]))

## ----lksess-------------------------------------------------------------------
sessionInfo()

#' collection of synthea FHIR documents ingested
#' @docType data
#' @examples
#' data("allin", package="BiocFHIR")
#' allin[[1]]
#' @format list
#' @usage data("allin", package = "BiocFHIR")
"allin"

## ----setup,results="hide",echo=FALSE------------------------------------------
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
})
})

## ----takepeek-----------------------------------------------------------------
peek = jsonlite::fromJSON(dir(system.file("json", package="BiocFHIR"), full=TRUE))
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))

## ----lkcli--------------------------------------------------------------------
table(peek$entry$resource$resourceType)

## ----lksess-------------------------------------------------------------------
sessionInfo()

#' extract information from retained fields in Observation component of FHIR Bundle, produce simple data.frame
#' @importFrom tidyr unnest
#' @param Observation component of FHIR.bundle instance
#' @return data.frame
#' @examples
#' testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
#'    package="BiocFHIR")
#' tbun <- process_fhir_bundle(testf)
#' process_Observation(tbun$Observation)
#' @export
process_Observation <- function(Observation) {
 stopifnot(inherits(Observation, "BiocFHIR.Observation"))
 last = function(x) x[nrow(x),]
 coding = lapply(Observation$code$coding, last)  # deal with, e.g., body temp/oral temp in one measurement
 coding <- do.call(rbind, coding)
 ans <- data.frame(id=Observation$id, subject.reference=Observation$subject$reference, 
     code.coding=coding, valueQuantity=Observation$valueQuantity, 
     effectiveDateTime=Observation$effectiveDateTime, issued=Observation$issued)
# bloodpressure records are nested in component
  fixbp <- function(component.element) {
    al <- as.list(component.element)
    data.frame(bp.text=al$code$text, bp.vals=al$valueQuantity)
  }
# get bp records
 isbp <- which(ans$code.coding.display == "Blood Pressure")
 if (length(isbp)>0) {
   bprecs <- ans[isbp,,drop=FALSE]
   ans <- ans[-isbp,,drop=FALSE]
   }
# get associated 'component' records, which have nested data.frame each with 2 rows
 allcomp <- Observation$component[isbp]

 fixedbp <- do.call(rbind, lapply(allcomp, fixbp))

# use new data.frames from fixbp and doubling of "Blood Pressure" record metadata
# to produce a conformant data.frame
 newid <- rep(bprecs$id, each=2)
 newsubjref <- rep(bprecs$subject.reference, each=2)
 newcodesys <- rep(bprecs$code.coding.system, each=2)
 newcodecod <- rep(bprecs$code.coding.code, each=2)
 newdatetime <- rep(bprecs$effectiveDateTime, each=2)
 newissued <- rep(bprecs$issued, each=2)
 newcodedisp <- fixedbp$bp.text
 newvqv <- fixedbp$bp.vals.value
 newvqu <- fixedbp$bp.vals.unit
 newvqs <- fixedbp$bp.vals.system

 revdf <- data.frame(id=newid, subject.reference=newsubjref,
   code.coding.system=newcodesys,
   code.coding.code=newcodecod,
   code.coding.display=newcodedisp,
   valueQuantity.value = newvqv,
   valueQuantity.unit = newvqu,
   valueQuantity.system = newvqs,
   valueQuantity.code = NA,
   effectiveDateTime = newdatetime,
   issued = newissued)

 ans <- rbind(ans, revdf) 
 ans
}

#> summary_selections
#function() c(AllergyIntolerance = "code.display", CarePlan = "category.display", 
#Claim = "insurance", Condition = "code.coding.display", Encounter = "type.display", 
#Immunization = "vaccineCode.display", MedicationRequest = "medicationCodeableConcept.display", 
#Observation = "code.coding.display", Patient = "id", Procedure="code.display"
#)
#<bytecode: 0x56243b038e18>
#> names(summary_selections())
# [1] "AllergyIntolerance" "CarePlan"           "Claim"             
# [4] "Condition"          "Encounter"          "Immunization"      
# [7] "MedicationRequest"  "Observation"        "Patient"           
#[10] "Procedure"  


#' produce interactive tables with FHIR resources from a list of ingested bundles
#' @import shiny
#' @param blist list of ingested bundles
#' @return side-effects of shiny app invocation
#' @examples
#' if (interactive()) {
#' tset = make_test_json_set()
#' bl = lapply(tset, process_fhir_bundle)
#' freq_app(bl)
#' }
#' @export
freq_app = function(blist) {
 ui = fluidPage(
  sidebarLayout(
   sidebarPanel(
    helpText("BiocFHIR bundle tabulator"),
    radioButtons("type", "type", choices=c("event", "patient", "provider", "center"), selected="event"),
    textOutput("desc"),
   width=2),
   mainPanel(
    tabsetPanel(
     tabPanel("Encounter",
      DT::dataTableOutput("encounter")),
     tabPanel("Condition",
      DT::dataTableOutput("cond")),
     tabPanel("Med",
      DT::dataTableOutput("medreq")),
     tabPanel("Procedure",
      DT::dataTableOutput("proc")),
     tabPanel("Allergy",
      DT::dataTableOutput("allint")),
     tabPanel("Immunization",
      DT::dataTableOutput("immuz")),
     tabPanel("About",
      helpText("Derived from run_synthea -p 10000")
      )
     )
    )
   )
  )

server = function(input, output, session) {
 output$desc = renderText(sprintf("%d bundles ingested", length(blist)))
 output$encounter = DT::renderDataTable(
  summarise_bundles(blist, "Encounter")
  )
 output$cond = DT::renderDataTable(
  summarise_bundles(blist, "Condition")
  )
 output$medreq = DT::renderDataTable(
  summarise_bundles(blist, "MedicationRequest")
  )
 output$proc = DT::renderDataTable(
  summarise_bundles(blist, "Procedure")
  )
 output$allint = DT::renderDataTable(
  summarise_bundles(blist, "AllergyIntolerance")
  )
 output$immuz = DT::renderDataTable(
  summarise_bundles(blist, "Immunization")
  )
 }
 
 runApp(list(ui=ui, server=server))
}
  
   

#' vector of fields to be selected for summarization
#' @return named vector of strings
#' @export
summary_selections = function() c(AllergyIntolerance = "code.display", CarePlan = "category.display", 
Claim = "insurance", Condition = "code.coding.display", Encounter = "type.display", 
Immunization = "vaccineCode.display", MedicationRequest = "medicationCodeableConcept.display", 
Observation = "code.coding.display", Patient = "id", Procedure="code.display"
)

#' produce tables summarizing FHIR data
#' @import dplyr
#' @param blist list of ingested bundles
#' @param resource character(1) FHIR resource name
#' @param selection_map character() named vector of single strings selected for summarisation
#' @return data.frame
#' @export
summarise_bundles = function(blist, resource="Condition", selection_map = summary_selections()) {
 st = stack_fhir(blist, resource)
 vbl = as.character(selection_map[resource])
 #st |> select(code.coding.display) |> group_by(code.coding.display) |> summarise(n=n())
 st |> select(!!sym(vbl)) |> group_by(!!sym(vbl)) |> summarise(n=n()) |> arrange(desc(n))
}

---
title: "BiocFHIR -- infrastructure for parsing and analyzing FHIR data"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{BiocFHIR -- infrastructure for parsing and analyzing FHIR data}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

```{r setup,results="hide",echo=FALSE}
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
})
})
```

# Introduction

FHIR stands for Fast Health Interoperability Resources.  

The [Wikipedia article](https://en.wikipedia.org/wiki/Fast_Healthcare_Interoperability_Resources)
is a useful overview.  The official website is [fhir.org](http://fhir.org).

This R package addresses very basic tasks of parsing FHIR R4 documents in JSON format.
The overall information model of FHIR documents is complex and various
decisions are made to help extract and annotate fields presumed to have
high value.  Submit github issues if important fields are not being
propagated.

Install this package using
```{r dobioc, eval=FALSE}
BiocManager::install("BiocFHIR")
```

## The basic structure of FHIR R4 JSON

We use `jsonlite::fromJSON` to import a randomly selected
FHIR document from a collection simulated by the MITRE corporation.
See the associated [site](https://synthea.mitre.org/downloads) for details.

We'll drill down through the hierarchy of elements collected in
a FHIR document with some base R commands, after importing the JSON.
```{r lkd1}
testf = dir(system.file("json", package="BiocFHIR"), full=TRUE)
tt = fromJSON(testf)
names(tt)
tt[1:2]
tte = tt$entry
class(tte)
dim(tte)
head(names(tte))
tter = tte$resource
dim(tter)
head(names(tter))
table(tter$resourceType)
```

It is by filtering the data frame `tter` that we acquire
information that may be useful in data analysis.  The
data frame is sparse: many fields are not used in many records.
Code in this package attempts to produce useful tables
from the sparse information.

As a prologue to table extraction, we do some basic
decomposition of `tter` using `process_fhir_bundle`.

```{r dobu1}
bu1 = process_fhir_bundle(testf) # just give file path
bu1
```

`bu1` is just a list of data.frames, but with considerable
nesting of data.frames and lists within the basic
data.frames corresponding to the major FHIR concepts.
"Flattening" of such structures is not fully automatic.

## Example: a table on Conditions recorded on the patient.

We use `process_Condition` to extract information.
```{r dopro1}
cond1 = process_Condition(bu1$Condition)
datatable(cond1)
```

## A family of documents

We have collected 50 documents from the synthea resource.
These were obtained using random draws from the 1180 records
provided.  A temporary folder holding them can be produced
as follows:

```{r doextr}
tset = make_test_json_set()
tset[1]
```

We import ten documents into a list.
```{r getalli}
myl = lapply(tset[1:10], process_fhir_bundle)
myl[1:2]
sapply(myl,length)
```
We see with the last command that documents can have different numbers
of components present.

# Session information

```{r lksess}
sessionInfo()
```

---
title: "Linking information between FHIR resources"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Linking information between FHIR resources}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

```{r setup,results="hide",echo=FALSE}
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
library(graph)
library(igraph)
})
})
```

# Introduction

The purpose of this vignette is to provide
details on how FHIR documents are
transformed to graphs in BiocFHIR.

This text uses R commands that will work for an R (version 4.2 or greater) in which
BiocFHIR (version 0.0.14 or greater) has been installed.  The source codes are
always available at [github](https://github.com/vjcitn/BiocFHIR) and may
be available for installation by other means.

# Examining sample data, again

In the "Upper level FHIR concepts" vignette, we used the
following code to get a peek at the information
structure in a single document representing a Bundle
associated with a patient.

```{r takepeek}
tfile = dir(system.file("json", package="BiocFHIR"), full=TRUE)
peek = jsonlite::fromJSON(tfile)
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))
```

We perform a first stage of transformation with `process_fhir_bundle`:
```{r txpeek}
bu = process_fhir_bundle(tfile)
bu
```

# A graph relating patients to conditions

```{r lkg1}
data(allin)
g = make_condition_graph(allin)
g
names(g)
```

We made a new S3 class to hold the graph with some convenient metadata.
Ultimately that metadata should be bound into the graph itself as nodeData
and edgeData components.

Because basic identifying information is decomposed into components
in FHIR, we have a utility to acquire the patient name for a given bundle.

```{r lkn}
getHumanName(allin[[1]]$Patient)
```

The edges emanating from the node corresponding to this patient are conditions that have
been recorded.  Edges are retrieved using the `edgeL` method.

```{r lke}
library(graph)
nodes(g$graph)[edgeL(g$graph)[["Ankunding277D'Amore443"]]$edges]
```

# Adding procedures to the graph

We have been unable so far to see how procedures
can be linked directly to conditions, except by
association with a given patient.  We
add the procedure information as follows:

```{r showa}
g = add_procedures(g, allin)
g
```

Data on additional resources can be added using
the methods of `add_procedures`.  This
will be carried out in future
releases.

# Interactive visualization of the graph

A visNetwork widget can be produced directly
from a list of ingested bundles.  This display can
be zoomed and dragged.  Procedures are green, patients
are blue, conditions are red.

```{r morep}
display_proccond_igraph( build_proccond_igraph( allin ))
```

# Conclusions

This collection of vignettes shows some approaches to
working with FHIR R4 JSON using R.  It is very likely that
a new collection of bundles obtained from a different source
would not be properly ingested or transformed by the code
present in this version of BiocFHIR.  Future extensions of
the package will employ direct analysis of JSON structures
to identify data values and relationships, that should be more
adaptable to diverse collections of documents.

Relationships among resources may be represented in many
different ways.  This survey of the resources in the synthea
bundles is surely limited, perhaps even with respect to the
information available in the bundles.  FHIR experts are invited
to identify gaps in this implementation.  We anticipate
considerable additional work needed to deal with other contexts
such as research studies.

# Session information

```{r lksess}
sessionInfo()
```

---
title: "Handling FHIR documents with BiocFHIR"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Handling FHIR documents with BiocFHIR}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

```{r setup,results="hide",echo=FALSE}
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
library(rjsoncons)
})
})
```

# Introduction

The purpose of this vignette is to provide
details on how FHIR documents are
handled in BiocFHIR.

This text uses R commands that will work for an R (version 4.2 or greater) in which
BiocFHIR (version 0.0.14 or greater) has been installed.  The source codes are
always available at [github](https://github.com/vjcitn/BiocFHIR) and may
be available for installation by other means.

We conclude this vignette with a very brief example of the
use of consjson to interrogate the FHIR JSON documents directly.

# Examining sample data, again

In the "Upper level FHIR concepts" vignette, we used the
following code to get a peek at the information
structure in a single document representing a Bundle
associated with a patient.

```{r takepeek}
tfile = dir(system.file("json", package="BiocFHIR"), full=TRUE)
peek = jsonlite::fromJSON(tfile)
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))
```

# Choosing an approach to FHIR JSON ingestion

Some of the complexity of working with FHIR JSON in R in this way
can be seen in the following:

```{r lkcat}
head(which(vapply(peek$entry$resource$category, 
   function(x)!is.null(x), logical(1))))
peek$entry$resource$category[[6]]
peek$entry$resource$category[[6]]$coding
peek$entry$resource$category[[10]]
```

Elements of category can be data.frame or atomic.
This is a consequence of naive use of `jsonlite::fromJSON`.

When we reduce the transformations attempted by `fromJSON`,
empty fields are not propagated.

```{r lkpeek2}
peek2 = jsonlite::fromJSON(dir(system.file("json", package="BiocFHIR"), full=TRUE), simplifyVector=FALSE)
lapply(peek2$entry[1:5], function(x) names(x[["resource"]]))
```

Because the JSON ingestion does not attempt to simplify
table-like content, we have a list of lists with varying
depths of nesting.

We can tabulate the resource types using
```{r lktab}
rtyvec = vapply(peek2$entry, function(x) 
   x[["resource"]]$resourceType, character(1))
table(rtyvec)
```

# Working with a specific type

## List-based operations

Let's use `peek2` to extract
Conditions recorded on the patient.

```{r getcond}
iscond = which(rtyvec == "Condition")
conds = peek2$entry[iscond]
length(conds)
str(conds[[1]])
```

Digging out the data and metadata on the first condition 
recorded (Perennial allergic rhinitis), is somewhat
complex using R.  Direct operations on JSON with JMESPATH
might be more effective, but we postpone this investigation.

## Processing with BiocFHIR

In the `process_fhir_bundle` function of BiocFHIR
we allow `jsonlite::fromJSON` to conduct some
simplification of list structures amenable to representation
as tables (data.frames).
```{r chkb}
tbu = process_fhir_bundle(tfile)
tbu
```

For the reports of Conditions, we extract specific
fields that are commonly used in the Synthea examples.
Other bundle sets may use different fields.

```{r lktab2}
ctab = process_Condition(tbu$Condition)
dim(ctab)
datatable(ctab)
```

The fields collected in `process_Condition` are specified
in `FHIR_retention_schemas()`.  Eventually this will
need to become a user-specified element of ingestion and
transformation.

# Direct querying of FHIR JSON

We've shown how we can operate on FHIR documents from the Synthea
project using specific schemas to select elements from lists
produced by parsing JSON.  The FHIR specification is very
flexible, and the `process_*` methods defined here may
not work for FHIR documents from other sources.

The jsoncons library provides C++ code for parsing and
filtering JSON, and the [rjsoncons](https://CRAN.R-project.org/package=rjsoncons)
package is available to support JMESPATH queries.

In this example, we'll take 4 Synthea FHIR documents and
extract patient addresses to a data.frame.

```{r dojme}
z = make_test_json_set()
myl = lapply(z[1:4], jsonlite::fromJSON) # list that rconsjson will convert to JSON
library(rjsoncons)
tmp = jmespath(myl, "[*].entry[0].resource.address") |> jsonlite::fromJSON()
do.call(rbind,lapply(tmp, function(x) x[,-(1:2)]))
```

The JMESPATH query projects from all documents via the initial `[*]`.
It then retrieves the address element from the resource element of
the first ([0]) entry.  An overview of the hierarchical structure of
`myl` can be obtained using `listviewer::jsonedit`.

# Session information

```{r lksess}
sessionInfo()
```

---
title: "Transforming FHIR documents to tables with BiocFHIR"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Transforming FHIR documents to tables with BiocFHIR}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

```{r setup,results="hide",echo=FALSE}
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
})
})
```

# Introduction

The purpose of this vignette is to provide
details on how FHIR documents are
transformed to tables in BiocFHIR.

This text uses R commands that will work for an R (version 4.2 or greater) in which
BiocFHIR (version 0.0.14 or greater) has been installed.  The source codes are
always available at [github](https://github.com/vjcitn/BiocFHIR) and may
be available for installation by other means.

# Examining sample data, again

In the "Upper level FHIR concepts" vignette, we used the
following code to get a peek at the information
structure in a single document representing a Bundle
associated with a patient.

```{r takepeek}
tfile = dir(system.file("json", package="BiocFHIR"), full=TRUE)
peek = jsonlite::fromJSON(tfile)
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))
```

We perform a first stage of transformation with `process_fhir_bundle`:
```{r txpeek}
bu = process_fhir_bundle(tfile)
bu
```

# Bundle to data frames

Each processed bundle is a collection of data.frame instances, formed
by splitting the input "entry" element by "resourceType".
These data.frames are mostly filled with NA missing values, but
some columns have been ingested as lists.  Executive decisions
are made in the package regarding which columns are likely
to hold useful information.  Thus we have

```{r lkpo}
po1 <- process_Observation(bu$Observation)
dim(po1)
datatable(po1)
```

# Filtering FHIR elements

A list of vectors of field names serves as the
basis for filtering JSON elements into
records for tabulation.
```{r lksch}
FHIR_retention_schemas()
```

Because each observation on Blood Pressure includes a "component"
element with two elements (for systolic and diastolic blood pressure readings),
special code is required to map the metadata for the Blood Pressure observations
to the specific values for each component.

# The resources extracted from a bundle

The `process_*` functions in BiocFHIR address various
resource types.  As of version 0.0.15 we have

```{r listt}
ls("package:BiocFHIR") |> grep(x=_, "process_[A-Z]", value=TRUE)
```

There is no guarantee that any given bundle with have
resources among all these types.

# Accumulating resources across bundles

Bundles are not guaranteed to have any specific
resources.  To assemble all information on
conditions recorded in the Synthea sample,
we must program defensively.  We obtain
the indices of bundles possessing a "Condition" resource,
and then combine the resulting tables, which are
designed to have a common set of columns.

```{r lkconds}
data("allin", package="BiocFHIR")
hascond = sapply(allin, function(x)length(x$Condition)>0)
oo = do.call(rbind, lapply(allin[hascond], function(x)process_Condition(x$Condition)))
dim(oo)
length(unique(oo$subject.reference))
```

The most commonly reported conditions in the sample are:
```{r mostc}
table(oo$code.coding.display) |> sort() |> tail()
```

# Session information

```{r lksess}
sessionInfo()
```

---
title: "Upper level FHIR concepts"
author: "Vincent J. Carey, stvjc at channing.harvard.edu"
date: "`r format(Sys.time(), '%B %d, %Y')`"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Upper level FHIR concepts}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    highlight: pygments
    number_sections: yes
    theme: united
    toc: yes
---

```{r setup,results="hide",echo=FALSE}
suppressPackageStartupMessages({
suppressMessages({
library(BiocFHIR)
library(DT)
library(jsonlite)
})
})
```

# Introduction

The purpose of this vignette is to explore
relationships between concepts underlying
FHIR foundational concepts and
aspects of implementation in BiocFHIR.

This text uses R commands that will work for an R (version 4.2 or greater) in which
BiocFHIR (version 0.0.14 or greater) has been installed.  The source codes are
always available at [github](https://github.com/vjcitn/BiocFHIR) and may
be available for installation by other means.

# Some comments from hl7.org/fhir Executive Summary

The following are verbatim from the [executive summary](http://hl7.org/fhir/summary.html)
on Oct 5 2022.  The FHIR system is said to provide

- A strong focus on implementation: fast and easy to implement (multiple developers have had simple interfaces working in a single day)
- Multiple implementation libraries, many examples available to kick-start development
- Specification is free for use with no restrictions
- Interoperability out-of-the-box: base resources can be used as is, but can also be adapted as needed - which happens a lot - for local requirements using Profiles, Extensions, Terminologies and more
- Evolutionary development path from HL7 Version 2 and CDA: standards can co-exist and leverage each other
- Strong foundation in Web standards: XML, JSON, HTTP, OAuth, etc.
- Support for RESTful architectures, seamless exchange of information using messages or documents, and service-based architectures
- Concise and easily understood specifications
- A human-readable serialization format for ease of use by developers
- Ontology-based analysis with formal mapping for correctness (under development)

End of quotation.

# FHIR JSON documents in BiocFHIR

The file located at `dir(system.file("json", package="BiocFHIR"), full=TRUE)`
is a single JSON document produced in the Synthea project of the MITRE
Corporation.  It is an extract from a zip file provided at the [Synthea web site](https://synthea.mitre.org/downloads).  Usage notes include:

> 'Data hosted within SyntheticMass has been generated by SyntheaTM, an open-source patient population simulation made available by The MITRE Corporation.

> The data is free from cost, privacy, and security restrictions. It can be used without restriction for a variety of secondary uses in academia, research, industry, and government.

> Please cite SyntheaTM or SyntheticMass as:

> Jason Walonoski, Mark Kramer, Joseph Nichols, Andre Quina, Chris Moesel, Dylan Hall, Carlton Duffett, Kudakwashe Dube, Thomas Gallagher, Scott McLachlan, Synthea: An approach, method, and software mechanism for generating synthetic patients and the synthetic electronic health care record, Journal of the American Medical Informatics Association, Volume 25, Issue 3, March 2018, Pages 230–238, https://doi.org/10.1093/jamia/ocx079'")

A collection of 50 documents, randomly sampled from over 1100 provides at Synthea,
is available using the command `BiocFHIR::make_test_json_set()`; the documents will
be written to a temporary folder.

# Examining sample data

```{r takepeek}
peek = jsonlite::fromJSON(dir(system.file("json", package="BiocFHIR"), full=TRUE))
names(peek)
peek$resourceType
names(peek$entry)
length(names(peek$entry$resource))
class(peek$entry$resource)
dim(peek$entry$resource)
head(names(peek$entry$resource))
```

There are many concepts covered in `peek$entry$resource`, but in this
document, most of them will have no content.

It is significant to note that the `resourceType` has value "Bundle".  The
meaning of this term is provided at [http://hl7.org/fhir/bundle.html](http://hl7.org/fhir/bundle.html).

Drilling down a bit further, we see that the 301 records
collected in this Bundle have resource types indicative of a clinical focus.
```{r lkcli}
table(peek$entry$resource$resourceType)
```
More details on these types
can be examined at [http://hl7.org/fhir/clinicalsummary-module.html](http://hl7.org/fhir/clinicalsummary-module.html).

# Session information

```{r lksess}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{add_procedures}
\alias{add_procedures}
\title{update a fhir graph on patients and conditions with procedures}
\usage{
add_procedures(fhirgraph, listOfProcessedBundles)
}
\arguments{
\item{fhirgraph}{instance of BiocFHIR.FHIRgraph}

\item{listOfProcessedBundles}{list}
}
\value{
instance of BiocFHIR.FHIRgraph
}
\description{
update a fhir graph on patients and conditions with procedures
}
\examples{
data("allin")
g <- make_condition_graph(allin)
g <- add_procedures(g, allin)
g
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/schemas.R
\name{FHIR_ResourceTypes}
\alias{FHIR_ResourceTypes}
\title{FHIR Resource types recognized in package}
\usage{
FHIR_ResourceTypes()
}
\value{
vector of strings
}
\description{
FHIR Resource types recognized in package
}
\examples{
FHIR_ResourceTypes() # Oct 2022
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{build_proccond_igraph}
\alias{build_proccond_igraph}
\title{build graph with patients, conditions and procedures}
\usage{
build_proccond_igraph(listOfBundles)
}
\arguments{
\item{listOfBundles}{list of processed FHIR bundles, processed with `process_fhir_bundle`}
}
\value{
instance of visIgraph from visNetworks
}
\description{
build graph with patients, conditions and procedures
}
\examples{
data("allin")
build_proccond_igraph( allin )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_Patient.R
\name{process_Patient}
\alias{process_Patient}
\title{flatten information in Patient component of a bundle to a one-line data.frame}
\usage{
process_Patient(Patient)
}
\arguments{
\item{Patient}{element of FHIR.bundle instance}
}
\value{
data.frame
}
\description{
flatten information in Patient component of a bundle to a one-line data.frame
}
\examples{
testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
   package="BiocFHIR")
tbun <- process_fhir_bundle(testf)
tpat <- process_Patient(tbun$Patient)
head(names(tpat))
tags <- c("identifier.system3", "identifier.value3")
tpat[tags,,FALSE]
tags2 <- grep("extension.extension", rownames(tpat), value=TRUE)
tpat[tags2,,FALSE]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{getHumanName}
\alias{getHumanName}
\title{get human name from a BiocFHIR.Patient instance}
\usage{
getHumanName(Patient)
}
\arguments{
\item{Patient}{BiocFHIR.Patient instance}
}
\value{
string with name components concatenated
}
\description{
get human name from a BiocFHIR.Patient instance
}
\examples{
testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
   package="BiocFHIR")
tbun <- process_fhir_bundle(testf)
getHumanName(tbun$Patient)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_support.R
\name{summary_selections}
\alias{summary_selections}
\title{vector of fields to be selected for summarization}
\usage{
summary_selections()
}
\value{
named vector of strings
}
\description{
vector of fields to be selected for summarization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_json.R
\name{available_retention_schemas}
\alias{available_retention_schemas}
\title{list available 'retention schemas'}
\usage{
available_retention_schemas()
}
\value{
character vector
}
\description{
list available 'retention schemas'
}
\examples{
available_retention_schemas()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq_app.R
\name{freq_app}
\alias{freq_app}
\title{produce interactive tables with FHIR resources from a list of ingested bundles}
\usage{
freq_app(blist)
}
\arguments{
\item{blist}{list of ingested bundles}
}
\value{
side-effects of shiny app invocation
}
\description{
produce interactive tables with FHIR resources from a list of ingested bundles
}
\examples{
if (interactive()) {
tset = make_test_json_set()
bl = lapply(tset, process_fhir_bundle)
freq_app(bl)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processOthers.R
\name{process_MedicationRequest}
\alias{process_MedicationRequest}
\title{extract information from retained fields in MedicationRequest component of FHIR Bundle, produce simple data.frame}
\usage{
process_MedicationRequest(MedicationRequest)
}
\arguments{
\item{MedicationRequest}{component of FHIR.bundle instance}
}
\value{
data.frame
}
\description{
extract information from retained fields in MedicationRequest component of FHIR Bundle, produce simple data.frame
}
\examples{
testf <- system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
   package="BiocFHIR")
tbun <- process_fhir_bundle(testf)
process_MedicationRequest(tbun$MedicationRequest)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_json.R
\name{print.FHIR.bundle}
\alias{print.FHIR.bundle}
\title{print method}
\usage{
\method{print}{FHIR.bundle}(x, ...)
}
\arguments{
\item{x}{BiocFHIR FHIR.bundle instance}

\item{\dots}{not used}
}
\value{
print method
}
\description{
print method
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stack_fhir.R
\name{stack_fhir}
\alias{stack_fhir}
\title{convert data of a given FHIR type in a list of bundles to a data.frame}
\usage{
stack_fhir(blist, type, droperr = TRUE)
}
\arguments{
\item{blist}{list of FHIR bundles imported with process_fhir_bundle}

\item{type}{character(1) type, in names(blist[[1]]), e.g.}

\item{droperr}{logical(1) exclude records for which process_[type] fails, defaults to TRUE}
}
\value{
data.frame
}
\description{
convert data of a given FHIR type in a list of bundles to a data.frame
}
\examples{
jj = make_test_json_set()
b2 = lapply(jj[1:2], process_fhir_bundle)
ss = stack_fhir(b2, "Procedure")
head(ss,2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_json.R
\name{process_fhir_bundle}
\alias{process_fhir_bundle}
\title{process a bundle of FHIR R4 JSON}
\usage{
process_fhir_bundle(json_file, schemas = FHIR_retention_schemas())
}
\arguments{
\item{json_file}{character(1) path to text in JSON format}

\item{schemas}{list of character vectors defining expected fields, defaults to FHIR_retention_schemas()}
}
\value{
instance of FHIR.bundle, extending list
}
\description{
process a bundle of FHIR R4 JSON
}
\note{
If one encounters the error "Element ... lacks field", the schemas argument can be modified by removing
the noted field from the schema.
}
\examples{
testf = system.file("json/Vince741_Rogahn59_6fa3d4ab-c0b6-424a-89d8-7d9105129296.json",
   package="BiocFHIR")
tbun = process_fhir_bundle(testf)
tbun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_support.R
\name{summarise_bundles}
\alias{summarise_bundles}
\title{produce tables summarizing FHIR data}
\usage{
summarise_bundles(
  blist,
  resource = "Condition",
  selection_map = summary_selections()
)
}
\arguments{
\item{blist}{list of ingested bundles}

\item{resource}{character(1) FHIR resource name}

\item{selection_map}{character() named vector of single strings selected for summarisation}
}
\value{
data.frame
}
\description{
produce tables summarizing FHIR data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{allin}
\alias{allin}
\title{collection of synthea FHIR documents ingested}
\format{
list
}
\usage{
data("allin", package = "BiocFHIR")
}
\description{
collection of synthea FHIR documents ingested
}
\examples{
data("allin", package="BiocFHIR")
allin[[1]]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{display_proccond_igraph}
\alias{display_proccond_igraph}
\title{make network visualization}
\usage{
display_proccond_igraph(igraph)
}
\arguments{
\item{igraph}{instance of igraph produced by `build_procond_igraph`}
}
\value{
visIGraph instance
}
\description{
make network visualization
}
\examples{
data("allin")
g <- build_proccond_igraph( allin ) 
if (interactive()) {
 display_proccond_igraph( g )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_test_json_set.R
\name{make_test_json_set}
\alias{make_test_json_set}
\title{produce 50 json FHIR files in a folder}
\usage{
make_test_json_set(target = paste0(tempdir(), "/jsontest"), reuse = TRUE)
}
\arguments{
\item{target}{character(1) a path, defaults to `jsontest` under `tempdir()`; the
contents of synthfhir.zip, in inst/zip of BiocFHIR, will be deposted there.}

\item{reuse}{logical(1) if TRUE, just use what is there, if folder already exists}
}
\value{
a vector of paths to FHIR JSON, invisibly
}
\description{
produce 50 json FHIR files in a folder
}
\examples{
z <- make_test_json_set()
z[1:3]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/schemas.R
\name{FHIR_retention_schemas}
\alias{FHIR_retention_schemas}
\title{collection of FHIR Resource components to be retained}
\usage{
FHIR_retention_schemas()
}
\value{
list of vectors of strings
}
\description{
collection of FHIR Resource components to be retained
}
\examples{
FHIR_retention_schemas() # Oct 2022
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{make_condition_graph}
\alias{make_condition_graph}
\title{create graph with links from patients to conditions}
\usage{
make_condition_graph(listOfProcessedBundles, keep_with_condition_only = TRUE)
}
\arguments{
\item{listOfProcessedBundles}{list with elements generated by `process_fhir_bundle`}

\item{keep_with_condition_only}{logical(1) omit bundles that lack a "Condition" element, defaults to TRUE}
}
\value{
instance of BiocFHIR.FHIRgraph
}
\description{
create graph with links from patients to conditions
}
\examples{
data("allin")
make_condition_graph(allin, TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{FHIRtabs}
\alias{FHIRtabs}
\title{table app}
\usage{
FHIRtabs()
}
\value{
No value returned.
}
\description{
table app
}
\examples{
if (interactive()) {
 FHIRtabs()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphit.R
\name{print.BiocFHIR.FHIRgraph}
\alias{print.BiocFHIR.FHIRgraph}
\title{show a combination of graph and patient attributes}
\usage{
\method{print}{BiocFHIR.FHIRgraph}(x, ...)
}
\arguments{
\item{x}{instance of BiocFHIR.FHIRgraph}

\item{\dots}{not used}
}
\value{
print method
}
\description{
show a combination of graph and patient attributes
}
\examples{
data("allin")
make_condition_graph(allin)
}
